/**
 * Copyright (C) 2012, BMW AG
 *
 * This file is part of GENIVI Project AudioManager.
 *
 * Contributions are licensed to the GENIVI Alliance under one or more
 * Contribution License Agreements.
 *
 * \copyright
 * This Source Code Form is subject to the terms of the
 * Mozilla Public License, v. 2.0. If a  copy of the MPL was not distributed with
 * this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 *
 * \author Christian Mueller, christian.ei.mueller@bmw.de BMW 2011,2012
 *
 * \file
 * For further information see http://www.genivi.org/.
 *
 * THIS CODE HAS BEEN GENERATED BY ENTERPRISE ARCHITECT GENIVI MODEL. PLEASE CHANGE ONLY IN ENTERPRISE ARCHITECT AND GENERATE AGAIN
 */
#if !defined(EA_B5ABC40A_CD0B_4c35_BB93_F4EA426C0B58__INCLUDED_)
#define EA_B5ABC40A_CD0B_4c35_BB93_F4EA426C0B58__INCLUDED_

namespace am {
	/**
	 * This enum classifies the format in which data is exchanged within a connection.
	 * The enum itself is project specific although there are some Genivi standard
	 * formats defined.
	 * @author Christian Mueller
	 * @created 28-Jun-2012 14:00:25
	 */
	enum am_ConnectionFormat_e
	{
		/**
		 * default
		 */
		CF_UNKNOWN = 0,
		/**
		 * plain mono
		 */
		CF_GENIVI_MONO = 1,
		/**
		 * stereo connection
		 */
		CF_GENIVI_STEREO = 2,
		/**
		 * analog connection
		 */
		CF_GENIVI_ANALOG = 3,
		/**
		 * automatic connection. 
		 */
		CF_GENIVI_AUTO = 4,

		CF_MAX
	};

	/**
	 * This enum gives the information about reason for reason for Source/Sink change
	 * @author Christian Mueller
	 * @created 28-Jun-2012 14:00:25
	 */
	enum am_AvailabilityReason_e
	{
		/**
		 * default
		 */
		AR_UNKNOWN = 0,
		/**
		 * the availability changed because an new media was entered 
		 */
		AR_GENIVI_NEWMEDIA = 1,
		/**
		 * the availability changed because the same media was entered
		 */
		AR_GENIVI_SAMEMEDIA = 2,
		/**
		 * the availability changed because there is no media
		 */
		AR_GENIVI_NOMEDIA = 3,
		/**
		 * the availability changed because of a temperature event
		 */
		AR_GENIVI_TEMPERATURE = 4,
		/**
		 * the availability changed because of a voltage event
		 */
		AR_GENIVI_VOLTAGE = 5,
		/**
		 * the availability changed because of fatal errors reading or accessing media
		 */
		AR_GENIVI_ERRORMEDIA = 6,
		AR_MAX
	};

	/**
	 * product specific identifier of property
	 * @author Christian Mueller
	 * @created 28-Jun-2012 14:00:25
	 */
	enum am_ClassProperty_e
	{
		/**
		 * default
		 */
		CP_UNKNOWN = 0,
		/**
		 * defines the source type of a source. Project specific, could be for example
		 * differentiation between interrupt source and main source.
		 */
		CP_ENTERTAINMENT = 1,
		/**
		 * defines the SINK_TYPE. Project specific
		 */
		CP_INTERRUPT_OVERLAY = 2,

		CP_INTERRUPT_SUPERSEEDING = 3,

		CP_PDC =4,

		CP_GONG = 5,

        CP_CROSSFADER = 6,

		CP_GENIVI_SOURCE_TYPE=7,

        CP_GENIVI_SINK_TYPE=8,

        CP_VIRTUAL_MAINSINK = 9,

        CP_INVISIBLE_SOURCE = 10,

        CP_PLAINSINK = 11,

        CP_MAINSINK = 12,

        CP_INVISIBLE_SINK = 13,

		CP_MAX
	};

	/**
	 * The given ramp types here are just a possiblity. for products, different ramp
	 * types can be defined here.
	 * It is in the responsibility of the product to make sure that the routing
	 * plugins are aware of the ramp types used.
	 * @author Christian Mueller
	 * @created 28-Jun-2012 14:00:25
	 */
	enum am_RampType_e
	{
		RAMP_UNKNOWN = 0,
		/**
		 * this ramp type triggers a direct setting of the value without a ramp
		 */
		RAMP_GENIVI_DIRECT = 1,
		/**
		 * This ramp type will set the volume as fast as possible.  
		 */
		RAMP_GENIVI_NO_PLOP = 2,
		RAMP_GENIVI_EXP_INV = 3,
		RAMP_GENIVI_LINEAR = 4,
		RAMP_GENIVI_EXP = 5,
		RAMP_MAX
	};

	/**
	 * sound properties. Within genivi only the standard properties are defined, for
	 * products these need to be extended.
	 * @author Christian Mueller
	 * @created 28-Jun-2012 14:00:25
	 */
	enum am_SoundPropertyType_e
	{
		/**
		 * default
		 */
		SP_UNKNOWN = 0,
		/**
		 * example treble value min =-10 max =10
		 */
		SP_EXAMPLE_TREBLE = 1,
		/**
		 * example mid value min =-10 max =10
		 */
		SP_EXAMPLE_MID = 2,
		/**
		 * example bass value min =-10 max =10
		 */
		SP_EXAMPLE_BASS = 3,

		SP_ENTER_SINKID = 4,

		SP_INTERR_SUPERSEEDINGID = 5,

		SP_INTERR_OVERLAYID = 6,

		SP_PRIORITY = 7,

		SP_MAX
	};

	/**
	 * Here are all SoundProperties that can be set via the CommandInterface. Product
	 * specific
	 * @author Christian Mueller
	 * @created 28-Jun-2012 14:00:25
	 */
	enum am_MainSoundPropertyType_e
	{
		/**
		 * default
		 */
		MSP_UNKNOWN = 0,
		/**
		 * example value between -10 and +10 
		 */
		MSP_EXAMPLE_TREBLE = 1,
		/**
		 * example value between -10 and +10 
		 */
		MSP_EXAMPLE_MID = 2,
		/**
		 * example value between -10 and +10 
		 */
		MSP_EXAMPLE_BASS = 3,

		MSP_SOURCE_TYPE = 4,

		MSP_MAX
	};

	/**
	 * describes the different system properties. Project specific
	 * @author Christian Mueller
	 * @created 28-Jun-2012 14:00:25
	 */
	enum am_SystemPropertyType_e
	{
		/**
		 * default
		 */
		SYP_UNKNOWN = 0,
		SYP_MAX
	};

	enum am_sourceType_e
	{
	    ST_MULTIMEDIA_LIBRARY = 0,
	    ST_TUNER = 1,
	    ST_AUX_STREAM = 2,
	    ST_MAX
	};

	/**
	 * gives the type of the Notification.
	 * @author q232968
	 * @version 1.0
	 * @created 05-Dez-2012 12:36:55
	 */
	enum am_NotificationType_e
	{
		NT_UNKNOWN = 0,
		NT_MAX
	};
}
#endif // !defined(EA_B5ABC40A_CD0B_4c35_BB93_F4EA426C0B58__INCLUDED_)
